import cloneDeep from 'lodash/cloneDeep'
export type Obj = { [key: string]: any }
export type ArrayObj = Array<Obj>

export interface SearchColumnsIn {
  itemType?: 'input' | 'select' | 'picker' // 表单渲染类型
  model?: string // 绑定值
  search__id?: string // 唯一标识
  treeTitle?: string // 搜索项设置显示title
  slotScope?: boolean // 是否是插槽
  sortSubscript?: number | undefined // 排序标识
  disableSearch?: boolean | undefined // 是否选中
  hideInSetting?: boolean | undefined // 搜索项是否不设置
  label?: string // label名称
  fieldProps?: { [key: string]: any } // 渲染参数
  list?: string[] // itemType是select时必传，[数组集合，label，value]
  listeners?: { [key: string]: Function } // 绑定方法
  disabledName?: string // 禁选标识（根据该标识的数组长度）
}

export type SearchColumnsType = Array<SearchColumnsIn>

// 添加search__id唯一标识
export const AddListSearchId = (attr: SearchColumnsType) => {
  attr.map(v => {
    v.search__id = Number(Math.random().toString().substr(3, 10) + Date.now()).toString(36)
  })
  return [...attr]
}

// 排序
export const ArraySort = (attr: any[], name: string): any[] => {
  const _attr = [...attr]
  _attr.sort((a, b) => {
    return a[name] - b[name]
  })
  return _attr
}

// 搜索项操作
export function SetSearchColumnsDate(
  data: SearchColumnsType | string[],
  type: string,
  params: Obj
): SearchColumnsType {
  const { columns, original_columns = [] } = cloneDeep(params)
  const current_columns = cloneDeep(columns)
  let return_columns: SearchColumnsType = []
  switch (type) {
    case 'reset':
      original_columns.map((v: SearchColumnsIn, k: number) => (v.sortSubscript = k))
      return_columns = [...original_columns]
      break
    case 'select':
      current_columns.map((v: SearchColumnsIn) => (data as string[]).some(m => m === v.search__id && !v.hideInSetting) ? (v.disableSearch = false) : '')
      return_columns = [...current_columns]
      break
    case 'del':
      current_columns.map((v: SearchColumnsIn) => (data as string[]).some(m => m === v.search__id && !v.hideInSetting) ? (v.disableSearch = true) : '')
      return_columns = [...current_columns]
      break
    case 'sort':
      (data as SearchColumnsType).map((v: Obj, k: number) => (v.sortSubscript = k))
      return_columns = [...(data as SearchColumnsType)]
      break
  }
  return return_columns
}
